@@ -30,6 +30,7 @@ from __future__ import print_function
 
 import argparse
 import cmd
+import logging
 import os
 import re
 import shlex
@@ -51,9 +52,10 @@ class ShellParser(argparse.ArgumentParser):
 class Shell(cmd.Cmd):
     curdir = '/'
 
-    def __init__(self, hosts=[]):
+    def __init__(self, hosts=[], timeout=10):
         cmd.Cmd.__init__(self)
         self._hosts = hosts
+        self._connect_timeout = timeout
         self._setup_readline()
 
         self._zk = None
@@ -66,6 +68,9 @@ class Shell(cmd.Cmd):
         if not self.connected:
             self._update_curdir('/')
 
+    def emptyline(self):
+        pass
+
     def run(self):
         self.cmdloop("")
 
@@ -76,6 +81,14 @@ class Shell(cmd.Cmd):
           print("Not connected.")
         return wrapped
 
+    def interruptible(f):
+        def wrapped(self, args):
+            try:
+                f(self, args)
+            except KeyboardInterrupt:
+                pass
+        return wrapped
+
     def ensure_params(expected_params):
         def wrapper(f):
             parser = ShellParser()
@@ -101,7 +114,7 @@ class Shell(cmd.Cmd):
     def check_path_exists(f):
         def wrapped(self, params):
             path = params.path
-            params.path = self._abspath(path if path != '' else self.curdir)
+            params.path = self._abspath(path if path not in ["", "."] else self.curdir)
             if self._zk.exists(params.path):
                 return f(self, params)
             print("Path %s doesn't exist" % (path))
@@ -261,19 +274,44 @@ example:
 """)
 
     @connected
-    @ensure_params([("path", False)])
+    @interruptible
+    @ensure_params([("path", False), ("max_depth", False)])
     @check_path_exists
     def do_tree(self, params):
+        max_depth = 0
+        try:
+            max_depth = int(params.max_depth) if params.max_depth != "" else 0
+        except ValueError: pass
+
         print(".")
-        self._print_tree(params.path, 0)
+        self._print_tree(params.path, 0, max_depth)
 
     def complete_tree(self, cmd_param_text, full_cmd, start_idx, end_idx):
         return self._complete_path(cmd_param_text, full_cmd)
 
-    def _print_tree(self, path, indent):
+    def help_tree(self):
+        print("""
+print the tree under a given path (optionally only up to a given max depth).
+
+example:
+  tree
+   .
+   ├── zookeeper
+   │   ├── config
+   │   ├── quota
+
+  tree 1
+   .
+   ├── zookeeper
+   ├── foo
+   ├── bar
+""")
+
+    def _print_tree(self, path, level, max_depth):
         for c in self._zk.get_children(path):
-            print(u"%s├── %s" % (u"│   " * indent, c))
-            self._print_tree(u"%s/%s" % (path, c), indent + 1)
+            print(u"%s├── %s" % (u"│   " * level, c))
+            if max_depth == 0 or level + 1 < max_depth:
+                self._print_tree(u"%s/%s" % (path, c), level + 1, max_depth)
 
     @connected
     @ensure_params([("path", True), ("match", True)])
@@ -611,8 +649,11 @@ example:
     def _connect(self, hosts):
         self._disconnect()
         self._zk = KazooClient(hosts=",".join(hosts), read_only=self._read_only)
-        self._zk.start()
-        self.connected = True
+        try:
+            self._zk.start(timeout=self._connect_timeout)
+            self.connected = True
+        except Exception as ex:
+            print("Failed to connect: %s" % (ex))
         self._update_curdir('/')
 
     def _update_curdir(self, dirpath):
